/*
 * fft.c
 *
 *  Created on: 29/05/2014
 *      Author: toni
 */
#include "LPC17xx.h"
#include "FreeRTOS.h"
#include "printf.h"
#include "semphr.h"
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "menu.h"
#include "adc.h"
#include "lpc_types.h"
#include "cr_dsplib.h"
#include <math.h>
#include "commonutils.h"
static xSemaphoreHandle FFTMutex = NULL;
extern volatile uint32_t ADC0Value[ADC_NUM];
extern volatile uint32_t ADC0IntDone;
extern void TargetResetInit(void);
uint32_t variable=0;

void FFTStart(void) {
	SystemCoreClockUpdate();
	FFTMutex = xSemaphoreCreateMutex();

}

/*****************************************************************************
 * Function:		FFTDadaAdc
 * Description:		Funció privada , que executa el sistema de lectura del ADC en mode interrupció
 ** parameters:			None
 ** Returned value:		0-4095
 **
 ** Nota: el process de conversió es fa a:
 ** ((CPU Clock)/4)/((ADC Clock)-1))<255 i ADC Clock a de ser <13Mhz
 ** Mostres/s=ADC Clock/65 on 65 són els cicles necessaris per completar una conversió
 *****************************************************************************/
uint16_t FFTDadaAdc(void) {
	//lectura ADC amb interrupcions

    ADCInit( ADC_CLK ); // Inicia el ADC
	ADC0Read( 0 ); // Executa la conversió
	while ( !ADC0IntDone ); // Un cop finalitza la conversió el flag de la interrupció s'activa
	ADC0IntDone = 0; // Estableix a 0 el flag fi interrupcio
	return ADC0Value[0]; //Retorna el valor del ADC

}
void FFTSScan(void) {
	// Assignació per bancs de memoria SRAM
	//Banc 0 de 0x2007 C000 a 0x2007 FFFF
	//Banc 1 de 0x2008 0000 a 0x2008 3FFF
	static short *mostres_entrada = (short *) 0x2007C000;
	static short *mostres_sortida = (short *) 0x20080000;

	volatile short int i;
	volatile short int magnitud = 0;

	//Mostregem 256 cops , opcions de 16,64,256,1024
	for (i = 0; i < 256; i++) {

		mostres_entrada[2 * i] =FFTDadaAdc();
		mostres_entrada[(2 * i) + 1] = 0; //la part real es 0
	}
	// Es passen les mostres a la llibreria DSP
	vF_dspl_fftR4b16N256(mostres_sortida, mostres_entrada);

	//Calcula la magnitud dels n valors pero sense calcular la magnitud de la continua , la continua es la mostra 0
	//Formata els valors per transmetrels com ha paquets de 11 bytes
	for (i = 1; i < 256; i += 1)
	{
		magnitud = (int) sqrt((pow(mostres_sortida[i], 2) + pow(mostres_sortida[(i) + 1], 2))); // magnitud = sqrt(a^2 + b^2)

		if (i == 255) {
			Printf_UART(WIFIPORT, "%04d,%06d\n", i, magnitud);
		} else {
			Printf_UART(WIFIPORT, "%04d,%06d", i, magnitud);
		}
		// retard per la transmissió necessari perque no sobreescrigui idependement de la velocitat
		// de conexio del wifi, 51 ms es el cas mes desfavorable
		// el temps de transmissió ha de ser de 13 s a 256 mostres

		commonutilsdelayMS(51);
	}

}
