/*
 FreeRTOS V6.1.1 - Copyright (C) 2011 Real Time Engineers Ltd.

 This file is part of the FreeRTOS distribution.

 FreeRTOS is free software; you can redistribute it and/or modify it under
 the terms of the GNU General Public License (version 2) as published by the
 Free Software Foundation AND MODIFIED BY the FreeRTOS exception.
 ***NOTE*** The exception to the GPL is included to allow you to distribute
 a combined work that includes FreeRTOS without being obliged to provide the
 source code for proprietary components outside of the FreeRTOS kernel.
 FreeRTOS is distributed in the hope that it will be useful, but WITHOUT
 ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
 more details. You should have received a copy of the GNU General Public
 License and the FreeRTOS license exception along with FreeRTOS; if not it
 can be viewed here: http://www.freertos.org/a00114.html and also obtained
 by writing to Richard Barry, contact details for whom are available on the
 FreeRTOS WEB site.

 1 tab == 4 spaces!

 http://www.FreeRTOS.org - Documentation, latest information, license and
 contact details.

 http://www.SafeRTOS.com - A version that is certified for use in safety
 critical systems.

 http://www.OpenRTOS.com - Commercial support, development, porting,
 licensing and training services.
 */

/* FreeRTOS.org includes. */

#include "FreeRTOS.h"
#include "LPC17xx.h"
#include "task.h"
#include "semphr.h"
#include "queue.h"
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "scanf.h"
#include "fft.h"
#include "analitzador.h"
#include <printf.h>
#include "config.h"

/* Used as a loop counter to create a very crude delay. */
#define mainDELAY_LOOP_COUNT		( 0xfffff )

/* xQueueHandle */
xQueueHandle xQueueDadaRebuda;

/* Mides Cua */
#define MIDACUA_DadesRebudes	5

/* The task functions. */
void vCommReadTask(void *pvParameters);
void vFFTTask(void *pvParameters);

/*-----------------------------------------------------------*/

/*****************************************************************************
** Function name:		main
**
** Descriptions:		Crea la cua i les tasques , fent us de AnalitzadorInici,
** 						inicialitza les UART,FFT i configura el Wifi
**
*****************************************************************************/
int main(void) {
	//Crea la cua
	xQueueDadaRebuda = xQueueCreate(MIDACUA_DadesRebudes, sizeof(int));

	//Inicialitza la UART,FFT i configura el WIFI , estableix la conexio
	while(!AnalitzadorInici());

	// Crea les task de rebuda de dades i processat de la FFT
	if (xQueueDadaRebuda != NULL) {
		xTaskCreate(vCommReadTask, "Task Lectura", 500, NULL,
				tskIDLE_PRIORITY + 2, NULL);
		xTaskCreate(vFFTTask, "Task FFT", 500, NULL, tskIDLE_PRIORITY + 2,
				NULL);
		vTaskStartScheduler();
	}
	for (;;);
	return 0;
}
/*-----------------------------------------------------------*/
/*****************************************************************************
** Task name:		vCommReadTask
**
** Descriptions:	Obte la comanda del servidor TCP i aquesta es posa a la cua
**
*****************************************************************************/
void vCommReadTask(void *pvParameters) {
	int ordre = 0;
	char rebut[200];
	for (;;) {
		if ((uxQueueMessagesWaiting(xQueueDadaRebuda) < MIDACUA_DadesRebudes)) {
			if (UART0State == 1) {
				uint8_t size;
				memset(rebut, 0x00, 200);
				size = scanf_uart0(rebut, 250);
				if (size > 0) {
					if (strstr(rebut, "initfft")) {
						Printf_UART(LOGPORT,
								"%s-- Comanda rebuda desde el servidor\r",
								rebut);
						Printf_UART(LOGPORT, "Inicia FFT\n\r");
						ordre = 1;
						// posa la comanda a la cua
						if (!xQueueSend(xQueueDadaRebuda, &ordre, 500)) {
							Printf_UART(LOGPORT,
									"|vCommReadTask| Error en colocar a la cua");
						}
					} else {
						Printf_UART(LOGPORT, "%s-- rebut, %d ---mida\n\r",
								rebut, size);
					}
				}
			}
		}
	}
}
/*****************************************************************************
** Task name:		vFFTTask
**
** Descriptions:	Obte la comanda de la cua , actualitza la velocitat del clock de la CPU
** 					suspen les tasques tret aquesta per tal de no desviar la velocitat de mostreig
** 					Inicia la FFT i retorna les tasques a funcionar
*****************************************************************************/
void vFFTTask(void *pvParameters) {
	int comanda;
	portBASE_TYPE xStatus;
	for (;;) {
		if (xQueueDadaRebuda != 0) {
			//S'obte la comanda de la cua si es que hi ha
			xStatus = xQueueReceive(xQueueDadaRebuda, &comanda, 1000);
			// Comprova si la extraccio de la cua es correcte i la comanda quina es
			if ((xStatus == 1) && (comanda == 1)) {
				Printf_UART(LOGPORT, "FFTScan |vFFTask|%d\n\r", comanda);
				SystemCoreClockUpdate(); // actualització del clock
				Printf_UART(LOGPORT, "%d--velocitat CPU\r\n", SystemCoreClock);
				vTaskSuspendAll();
				FFTSScan(); // Recull de les dades i execucció de la FFT
				xTaskResumeAll();
			}
		}
	}

}

/*-----------------------------------------------------------*/

/*-----------------------------------------------------------*/

void vApplicationMallocFailedHook(void) {
	/* This function will only be called if an API call to create a task, queue
	 or semaphore fails because there is too little heap RAM remaining. */
	for (;;)
		;
}
/*-----------------------------------------------------------*/

void vApplicationStackOverflowHook(xTaskHandle *pxTask, signed char *pcTaskName) {
	/* This function will only be called if a task overflows its stack.  Note
	 that stack overflow checking does slow down the context switch
	 implementation. */
	for (;;)
		;
}
/*-----------------------------------------------------------*/

void vApplicationIdleHook(void) {
	/* This example does not use the idle hook to perform any processing. */
}
/*-----------------------------------------------------------*/

void vApplicationTickHook(void) {
	/* This example does not use the tick hook to perform any processing. */
}

